@@ -8,7 +8,7 @@
  *
  *    In some countries OX, OX Open-Xchange, open xchange and OXtender
  *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the Open-Xchange, Inc. group of companies.
+ *    trademarks of the OX Software GmbH group of companies.
  *    The use of the Logos is not covered by the GNU General Public License.
  *    Instead, you are allowed to use these Logos according to the terms and
  *    conditions of the Creative Commons License, Version 2.5, Attribution,
@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2006 Open-Xchange, Inc.
+ *     Copyright (C) 2016-2020 OX Software GmbH
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,21 +50,21 @@
 package com.openexchange.imap.cache;
 
 import static com.openexchange.imap.IMAPCommandsCollection.performCommand;
-import gnu.trove.map.hash.TObjectIntHashMap;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 import java.util.TreeMap;
 import java.util.TreeSet;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.atomic.AtomicBoolean;
 import javax.mail.Folder;
@@ -76,6 +76,7 @@ import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCommandsCollection;
 import com.openexchange.imap.config.IMAPReloadable;
 import com.openexchange.imap.services.Services;
+import com.openexchange.java.Strings;
 import com.openexchange.mail.mime.MimeMailException;
 import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.ProtocolException;
@@ -88,6 +89,7 @@ import com.sun.mail.imap.protocol.BASE64MailboxDecoder;
 import com.sun.mail.imap.protocol.BASE64MailboxEncoder;
 import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.IMAPResponse;
+import gnu.trove.map.hash.TObjectIntHashMap;
 
 /**
  * {@link ListLsubCollection}
@@ -113,10 +115,10 @@ final class ListLsubCollection implements Serializable {
     private final String[] user;
     private Boolean mbox;
     private long stamp;
-    private ListLsubEntryImpl draftsEntry;
-    private ListLsubEntryImpl junkEntry;
-    private ListLsubEntryImpl sentEntry;
-    private ListLsubEntryImpl trashEntry;
+    private final ConcurrentMap<String, ListLsubEntry> draftsEntries;
+    private final ConcurrentMap<String, ListLsubEntry> junkEntries;
+    private final ConcurrentMap<String, ListLsubEntry> sentEntries;
+    private final ConcurrentMap<String, ListLsubEntry> trashEntries;
 
     /**
      * Initializes a new {@link ListLsubCollection}.
@@ -130,8 +132,12 @@ final class ListLsubCollection implements Serializable {
      */
     protected ListLsubCollection(final IMAPFolder imapFolder, final String[] shared, final String[] user, final boolean doStatus, final boolean doGetAcl) throws MessagingException {
         super();
-        listMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
-        lsubMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
+        listMap = new ConcurrentHashMap<String, ListLsubEntryImpl>();
+        lsubMap = new ConcurrentHashMap<String, ListLsubEntryImpl>();
+        draftsEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        junkEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        sentEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        trashEntries = new ConcurrentHashMap<String, ListLsubEntry>();
         deprecated = new AtomicBoolean();
         this.shared = shared == null ? new String[0] : shared;
         this.user = user == null ? new String[0] : user;
@@ -152,6 +158,10 @@ final class ListLsubCollection implements Serializable {
         super();
         listMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
         lsubMap = new NonBlockingHashMap<String, ListLsubEntryImpl>();
+        draftsEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        junkEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        sentEntries = new ConcurrentHashMap<String, ListLsubEntry>();
+        trashEntries = new ConcurrentHashMap<String, ListLsubEntry>();
         deprecated = new AtomicBoolean();
         this.shared = shared == null ? new String[0] : shared;
         this.user = user == null ? new String[0] : user;
@@ -264,6 +274,68 @@ final class ListLsubCollection implements Serializable {
         }
     }
 
+    /**
+     * Re-Initializes the SPECIAL-USE folders (only if the IMAP store advertises support for <code>"SPECIAL-USE"</code> capability)
+     *
+     * @param imapStore The IMAP store
+     * @throws OXException If re-initialization fails
+     */
+    public void reinitSpecialUseFolders(final IMAPStore imapStore) throws OXException {
+        try {
+            reinitSpecialUseFolders((IMAPFolder) imapStore.getFolder("INBOX"), imapStore);
+        } catch (MessagingException e) {
+            throw MimeMailException.handleMessagingException(e);
+        }
+    }
+
+    /**
+     * Re-Initializes the SPECIAL-USE folders (only if the IMAP store advertises support for <code>"SPECIAL-USE"</code> capability)
+     *
+     * @param imapFolder The IMAP store
+     * @throws OXException If re-initialization fails
+     */
+    public void reinitSpecialUseFolders(final IMAPFolder imapFolder) throws OXException {
+        reinitSpecialUseFolders(imapFolder, (IMAPStore) imapFolder.getStore());
+    }
+
+    private void reinitSpecialUseFolders(final IMAPFolder imapFolder, final IMAPStore imapStore) throws OXException {
+        try {
+            draftsEntries.clear();
+            junkEntries.clear();
+            sentEntries.clear();
+            trashEntries.clear();
+            if (imapStore.getCapabilities().containsKey("SPECIAL-USE")) {
+                /*
+                 * Perform LIST (SPECIAL-USE) "" "*"
+                 */
+                imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
+
+                    @Override
+                    public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
+                        doListSpecialUse(protocol, true);
+                        return null;
+                    }
+
+                });
+            } else {
+                /*
+                 * Perform LIST "" "*"
+                 */
+                imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
+
+                    @Override
+                    public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
+                        doListSpecialUse(protocol, false);
+                        return null;
+                    }
+
+                });
+            }
+        } catch (MessagingException e) {
+            throw MimeMailException.handleMessagingException(e);
+        }
+    }
+
     /**
      * Re-initializes this collection.
      *
@@ -348,6 +420,10 @@ final class ListLsubCollection implements Serializable {
         if (clearMaps) {
             listMap.clear();
             lsubMap.clear();
+            draftsEntries.clear();
+            junkEntries.clear();
+            sentEntries.clear();
+            trashEntries.clear();
         }
         final boolean debug = LOG.isDebugEnabled();
         final long st = debug ? System.currentTimeMillis() : 0L;
@@ -395,7 +471,7 @@ final class ListLsubCollection implements Serializable {
 
                 @Override
                 public Object doCommand(final IMAPProtocol protocol) throws ProtocolException {
-                    doListSpecialUse(protocol);
+                    doListSpecialUse(protocol, true);
                     return null;
                 }
 
@@ -738,49 +814,46 @@ final class ListLsubCollection implements Serializable {
         "\\noselect",
         "\\hasnochildren")));
 
-    private static final Set<String> ATTRIBUTES_NO_SELECT_NAMESPACE = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("\\noselect")));
+    // private static final Set<String> ATTRIBUTES_NO_SELECT_NAMESPACE = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("\\noselect")));
 
     private static final String ATTRIBUTE_DRAFTS = "\\drafts";
     private static final String ATTRIBUTE_JUNK = "\\junk";
     private static final String ATTRIBUTE_SENT = "\\sent";
     private static final String ATTRIBUTE_TRASH = "\\trash";
 
+    private static final String[] ATTRIBUTES_SPECIAL_USE = { ATTRIBUTE_DRAFTS, ATTRIBUTE_JUNK, ATTRIBUTE_SENT, ATTRIBUTE_TRASH };
+
     /**
-     * Lists special folders via <code>LIST (SPECIAL-USE) "" "*"</code>.
+     * Performs a LIST/LSUB command with specified IMAP protocol.
      *
-     * @param protocol The associated IMAP protocol
+     * @param protocol The IMAP protocol
+     * @param lsub <code>true</code> to perform a LSUB command; otherwise <code>false</code> for LIST
      * @throws ProtocolException If a protocol error occurs
      */
-    protected void doListSpecialUse(final IMAPProtocol protocol) throws ProtocolException {
-        /*
-         * Perform command
-         */
-        final String command = "LIST";
-        final Response[] r;
-        {
-            final String sCmd = new StringBuilder(command).append(" (SPECIAL-USE) \"\" \"*\"").toString();
-            r = performCommand(protocol, sCmd);
-            LOG.debug("{0} cache filled with >>{}<< which returned {} response line(s).", (command), sCmd, r.length);
-        }
-        final Response response = r[r.length - 1];
+    protected void doListSpecialUse(final IMAPProtocol protocol, final boolean usingSpecualUse) throws ProtocolException {
+        String command = "LIST";
+        Response[] r = performCommand(protocol, new StringBuilder(command).append(usingSpecualUse ? " (SPECIAL-USE) " : " ").append("\"\" \"*\"").toString());
+        Response response = r[r.length - 1];
         if (response.isOK()) {
             for (int i = 0, len = r.length - 1; i < len; i++) {
                 if (!(r[i] instanceof IMAPResponse)) {
                     continue;
                 }
-                final IMAPResponse ir = (IMAPResponse) r[i];
+                IMAPResponse ir = (IMAPResponse) r[i];
                 if (ir.keyEquals(command)) {
-                    final ListLsubEntryImpl entryImpl = parseListResponse(ir, lsubMap);
-                    final Set<String> attrs = entryImpl.getAttributes();
-                    if (null != attrs && !attrs.isEmpty()) {
-                        if (attrs.contains(ATTRIBUTE_DRAFTS)) {
-                            this.draftsEntry = entryImpl;
-                        } else if (attrs.contains(ATTRIBUTE_JUNK)) {
-                            this.junkEntry = entryImpl;
-                        } else if (attrs.contains(ATTRIBUTE_SENT)) {
-                            this.sentEntry = entryImpl;
-                        } else if (attrs.contains(ATTRIBUTE_TRASH)) {
-                            this.trashEntry = entryImpl;
+                    ListLsubEntryImpl entryImpl = parseListResponse(ir, lsubMap, ATTRIBUTES_SPECIAL_USE);
+                    if (null != entryImpl) {
+                        Set<String> attrs = entryImpl.getAttributes();
+                        if (null != attrs && !attrs.isEmpty()) {
+                            if (attrs.contains(ATTRIBUTE_DRAFTS)) {
+                                this.draftsEntries.put(entryImpl.getFullName(), entryImpl);
+                            } else if (attrs.contains(ATTRIBUTE_JUNK)) {
+                                this.junkEntries.put(entryImpl.getFullName(), entryImpl);
+                            } else if (attrs.contains(ATTRIBUTE_SENT)) {
+                                this.sentEntries.put(entryImpl.getFullName(), entryImpl);
+                            } else if (attrs.contains(ATTRIBUTE_TRASH)) {
+                                this.trashEntries.put(entryImpl.getFullName(), entryImpl);
+                            }
                         }
                     }
                     r[i] = null;
@@ -810,7 +883,7 @@ final class ListLsubCollection implements Serializable {
         {
             final String sCmd = new StringBuilder(command).append(" \"\" \"*\"").toString();
             r = performCommand(protocol, sCmd);
-            LOG.debug("{} cache filled with >>{}<< which returned {} response line(s).", (command), sCmd, r.length);
+            LOG.debug("{} cache filled with >>{}<< which returned {} response line(s).", (command), sCmd, Integer.valueOf(r.length));
         }
         final Response response = r[r.length - 1];
         if (response.isOK()) {
@@ -867,6 +940,22 @@ final class ListLsubCollection implements Serializable {
                     listLsubEntry.setParent(rootEntry);
                     rootEntry.addChild(listLsubEntry);
                 }
+
+                // Check attributes for marked folder
+                if (false == lsub) {
+                    Set<String> attrs = listLsubEntry.getAttributes();
+                    if (null != attrs && !attrs.isEmpty()) {
+                        if (attrs.contains(ATTRIBUTE_DRAFTS)) {
+                            this.draftsEntries.put(listLsubEntry.getFullName(), listLsubEntry);
+                        } else if (attrs.contains(ATTRIBUTE_JUNK)) {
+                            this.junkEntries.put(listLsubEntry.getFullName(), listLsubEntry);
+                        } else if (attrs.contains(ATTRIBUTE_SENT)) {
+                            this.sentEntries.put(listLsubEntry.getFullName(), listLsubEntry);
+                        } else if (attrs.contains(ATTRIBUTE_TRASH)) {
+                            this.trashEntries.put(listLsubEntry.getFullName(), listLsubEntry);
+                        }
+                    }
+                }
             } // End of for loop
             if (!parentMap.isEmpty()) {
                 // Handle parent map
@@ -1046,7 +1135,7 @@ final class ListLsubCollection implements Serializable {
                 }
                 final IMAPResponse ir = (IMAPResponse) r[i];
                 if (ir.keyEquals(cmd)) {
-                    final ListLsubEntryImpl listLsubEntry = parseListResponse(ir, null);
+                    final ListLsubEntryImpl listLsubEntry = parseListResponse(ir, null, null, ROOT_FULL_NAME);
                     {
                         final ListLsubEntryImpl oldEntry = listMap.get(ROOT_FULL_NAME);
                         if (null == oldEntry) {
@@ -1164,6 +1253,120 @@ final class ListLsubCollection implements Serializable {
         return false; // Never reached if response is not OK
     }
 
+    /**
+     * Adds single folder to collection.
+     *
+     * @param imapFolder The IMAP folder
+     * @param addIt Whether to add the folder or not
+     * @throws OXException If operation fails
+     */
+    public ListLsubEntry addSingleByFolder(IMAPFolder imapFolder, boolean addIt) throws OXException {
+        try {
+            ListLsubEntry.ChangeState changeState = ListLsubEntry.ChangeState.UNDEFINED;
+            boolean canOpen = true;
+            boolean hasInferiors = true;
+            Boolean hasChildren = null;
+            Set<String> attributes;
+            {
+                String[] attrs = imapFolder.getAttributes();
+                if (null == attrs || 0 == attrs.length) {
+                    attributes = Collections.<String> emptySet();
+                } else {
+                    attributes = new HashSet<String>(attrs.length);
+                    for (String attribute : attrs) {
+                        String attr = Strings.asciiLowerCase(attribute);
+                        switch (POS_MAP.get(attr)) {
+                            case 1:
+                                changeState = ListLsubEntry.ChangeState.CHANGED;
+                                break;
+                            case 2:
+                                changeState = ListLsubEntry.ChangeState.UNCHANGED;
+                                break;
+                            case 3:
+                                canOpen = false;
+                                break;
+                            case 4:
+                                hasInferiors = false;
+                                break;
+                            case 5:
+                                hasChildren = Boolean.TRUE;
+                                break;
+                            case 6:
+                                hasChildren = Boolean.FALSE;
+                                break;
+                            default:
+                                // Nothing
+                                break;
+                            }
+                        attributes.add(attr);
+                    }
+                }
+            }
+            boolean subscribed = addIt ? imapFolder.isSubscribed() : false;
+            String fullName = imapFolder.getFullName();
+            if (subscribed) {
+                ListLsubEntryImpl lsubEntry = new ListLsubEntryImpl(fullName, attributes, imapFolder.getSeparator(), changeState, hasInferiors, canOpen, hasChildren, null);
+                ConcurrentMap<String, ListLsubEntryImpl> map = lsubMap;
+                {
+                    ListLsubEntryImpl oldEntry = map.get(fullName);
+                    ListLsubEntryImpl parent;
+                    if (null != oldEntry) {
+                        for (ListLsubEntryImpl child : oldEntry.getChildrenSet()) {
+                            child.setParent(lsubEntry);
+                            lsubEntry.addChild(child);
+                        }
+                        parent = oldEntry.getParentImpl();
+                    } else {
+                        int pos = fullName.lastIndexOf(lsubEntry.getSeparator());
+                        if (pos > 0) {
+                            String parentFullName = fullName.substring(0, pos);
+                            parent = map.get(parentFullName);
+                        } else {
+                            parent = map.get(ROOT_FULL_NAME);
+                        }
+                    }
+                    if (null != parent) {
+                        lsubEntry.setParent(parent);
+                        parent.addChild(lsubEntry);
+                    }
+                }
+                map.put(fullName, lsubEntry);
+            }
+
+            ListLsubEntryImpl listEntry = new ListLsubEntryImpl(fullName, attributes, imapFolder.getSeparator(), changeState, hasInferiors, canOpen, hasChildren, lsubMap);
+            if (addIt) {
+                ConcurrentMap<String, ListLsubEntryImpl> map = listMap;
+                {
+                    ListLsubEntryImpl oldEntry = map.get(fullName);
+                    ListLsubEntryImpl parent;
+                    if (null != oldEntry) {
+                        for (ListLsubEntryImpl child : oldEntry.getChildrenSet()) {
+                            child.setParent(listEntry);
+                            listEntry.addChild(child);
+                        }
+                        parent = oldEntry.getParentImpl();
+                    } else {
+                        int pos = fullName.lastIndexOf(listEntry.getSeparator());
+                        if (pos > 0) {
+                            String parentFullName = fullName.substring(0, pos);
+                            parent = map.get(parentFullName);
+                        } else {
+                            parent = map.get(ROOT_FULL_NAME);
+                        }
+                    }
+                    if (null != parent) {
+                        listEntry.setParent(parent);
+                        parent.addChild(listEntry);
+                    }
+                }
+                map.put(fullName, listEntry);
+            }
+            return listEntry;
+        } catch (MessagingException e) {
+            throw MimeMailException.handleMessagingException(e);
+        }
+    }
+
     /**
      * Adds single entry to collection.
      *
@@ -1403,47 +1606,47 @@ final class ListLsubCollection implements Serializable {
     }
 
     /**
-     * Gets the LIST entry marked with "\Drafts" attribute.
+     * Gets the LIST entries marked with "\Drafts" attribute.
      * <p>
      * Needs the <code>"SPECIAL-USE"</code> capability.
      *
-     * @return The entry or <code>null</code>
+     * @return The entries
      */
-    public ListLsubEntryImpl getDraftsEntry() {
-        return draftsEntry;
+    public Collection<ListLsubEntry> getDraftsEntry() {
+        return Collections.unmodifiableCollection(draftsEntries.values());
     }
 
     /**
-     * Gets the LIST entry marked with "\Junk" attribute.
+     * Gets the LIST entries marked with "\Junk" attribute.
      * <p>
      * Needs the <code>"SPECIAL-USE"</code> capability.
      *
-     * @return The entry or <code>null</code>
+     * @return The entries
      */
-    public ListLsubEntryImpl getJunkEntry() {
-        return junkEntry;
+    public Collection<ListLsubEntry> getJunkEntry() {
+        return Collections.unmodifiableCollection(junkEntries.values());
     }
 
     /**
-     * Gets the LIST entry marked with "\Sent" attribute.
+     * Gets the LIST entries marked with "\Sent" attribute.
      * <p>
      * Needs the <code>"SPECIAL-USE"</code> capability.
      *
-     * @return The entry or <code>null</code>
+     * @return The entries
      */
-    public ListLsubEntryImpl getSentEntry() {
-        return sentEntry;
+    public Collection<ListLsubEntry> getSentEntry() {
+        return Collections.unmodifiableCollection(sentEntries.values());
     }
 
     /**
-     * Gets the LIST entry marked with "\Trash" attribute.
+     * Gets the LIST entries marked with "\Trash" attribute.
      * <p>
      * Needs the <code>"SPECIAL-USE"</code> capability.
      *
-     * @return The entry or <code>null</code>
+     * @return The entries
      */
-    public ListLsubEntryImpl getTrashEntry() {
-        return trashEntry;
+    public Collection<ListLsubEntry> getTrashEntry() {
+        return Collections.unmodifiableCollection(trashEntries.values());
     }
 
     /**
@@ -1507,7 +1710,15 @@ final class ListLsubCollection implements Serializable {
         POS_MAP.put("\\hasnochildren", 6);
     }
 
-    private ListLsubEntryImpl parseListResponse(final IMAPResponse listResponse, final ConcurrentMap<String, ListLsubEntryImpl> lsubMap) {
+    private ListLsubEntryImpl parseListResponse(IMAPResponse listResponse, ConcurrentMap<String, ListLsubEntryImpl> lsubMap) {
+        return parseListResponse(listResponse, lsubMap, null);
+    }
+
+    private ListLsubEntryImpl parseListResponse(IMAPResponse listResponse, ConcurrentMap<String, ListLsubEntryImpl> lsubMap, String[] requiredAttributes) {
+        return parseListResponse(listResponse, lsubMap, requiredAttributes, null);
+    }
+
+    private ListLsubEntryImpl parseListResponse(IMAPResponse listResponse, ConcurrentMap<String, ListLsubEntryImpl> lsubMap, String[] requiredAttributes, String predefinedName) {
         /*
          * LIST (\NoInferiors \UnMarked) "/" "Sent Items"
          */
@@ -1524,9 +1735,14 @@ final class ListLsubCollection implements Serializable {
             /*
              * Non-empty attribute list
              */
+            if ((s.length <= 0) && (null != requiredAttributes)) {
+                // Cannot contain any required attribute
+                return null;
+            }
+
             attributes = new HashSet<String>(s.length);
-            for (int i = 0; i < s.length; i++) {
-                final String attr = s[i].toLowerCase(Locale.US);
+            for (int i = s.length; i-- > 0;) {
+                String attr = Strings.asciiLowerCase(s[i]);
                 switch (POS_MAP.get(attr)) {
                 case 1:
                     changeState = ListLsubEntry.ChangeState.CHANGED;
@@ -1554,6 +1770,25 @@ final class ListLsubCollection implements Serializable {
             }
         } else {
             attributes = Collections.emptySet();
+            if (null != requiredAttributes) {
+                // Cannot contain any required attribute
+                return null;
+            }
+        }
+        /*
+         * Check against required attributes
+         */
+        if (null != requiredAttributes) {
+            boolean containsAny = false;
+            for (String requiredAttribute : requiredAttributes) {
+                if (attributes.contains(requiredAttribute)) {
+                    containsAny = true;
+                    break;
+                }
+            }
+            if (!containsAny) {
+                return null;
+            }
         }
         /*
          * Read separator character
@@ -1575,7 +1810,7 @@ final class ListLsubCollection implements Serializable {
          * Read full name; decode the name (using RFC2060's modified UTF7)
          */
         listResponse.skipSpaces();
-        final String name = BASE64MailboxDecoder.decode(listResponse.readAtomString());
+        String name = null == predefinedName ? BASE64MailboxDecoder.decode(listResponse.readAtomString()) : predefinedName;
         /*
          * Return
          */
