@@ -49,9 +49,6 @@
 
 package com.openexchange.imap.command;
 
-import gnu.trove.impl.Constants;
-import gnu.trove.map.hash.TIntIntHashMap;
-import gnu.trove.map.hash.TLongIntHashMap;
 import java.io.IOException;
 import java.io.InputStream;
 import java.text.ParseException;
@@ -82,6 +79,7 @@ import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.ContentType;
 import com.openexchange.mail.mime.MessageHeaders;
 import com.openexchange.mail.mime.MimeMailException;
+import com.openexchange.mail.mime.PlainTextAddress;
 import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.mime.converters.MimeMessageConverter;
 import com.openexchange.mail.mime.utils.MimeMessageUtility;
@@ -98,7 +96,9 @@ import com.sun.mail.imap.protocol.Item;
 import com.sun.mail.imap.protocol.RFC822DATA;
 import com.sun.mail.imap.protocol.RFC822SIZE;
 import com.sun.mail.imap.protocol.UID;
-import com.sun.mail.imap.protocol.X_REAL_UID;
+import gnu.trove.impl.Constants;
+import gnu.trove.map.hash.TIntIntHashMap;
+import gnu.trove.map.hash.TLongIntHashMap;
 
 /**
  * {@link MailMessageFetchIMAPCommand} - performs a prefetch of messages in given folder with only those fields set that need to be present for
@@ -923,7 +923,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             msg.setSentDate(env.date);
         }
 
-        private InternetAddress[] wrap(InternetAddress... addresses) {
+        private InternetAddress[] wrap(final InternetAddress... addresses) {
             if (null == addresses || 0 == addresses.length) {
                 return null;
             }
@@ -931,11 +931,13 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             int length = addresses.length;
             InternetAddress[] ret = new InternetAddress[length];
             for (int i = length; i-- > 0;) {
+                String sAddress = addresses[i].toString();
                 try {
-                    ret[i] = new QuotedInternetAddress(addresses[i].toString());
+                    ret[i] = new QuotedInternetAddress(sAddress, false);
                 } catch (AddressException e) {
                     // Use as-is
-                    ret[i] = addresses[i];
+                    String parsed = e.getRef();
+                    ret[i] = new PlainTextAddress(null == parsed ? sAddress : parsed);
                 }
             }
             return ret;
