@@ -60,6 +60,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.ReentrantLock;
 import javax.mail.MessagingException;
+import com.openexchange.log.LogProperties;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.IMAPStore;
 
@@ -113,6 +114,11 @@ public class UnboundedIMAPStoreContainer extends AbstractIMAPStoreContainer {
             LOG.debug("UnboundedIMAPStoreContainer.getStore(): Returning newly established IMAPStore instance. {} -- {}", imapStore.toString(), imapStore.hashCode());
         } else {
             imapStore = imapStoreWrapper.imapStore;
+            // Grab associated IMAP session identifier (as advertised via "ID" command)
+            String sessionInformation = imapStore.getGeneratedExternalId();
+            if (null != sessionInformation) {
+                LogProperties.put(LogProperties.Name.MAIL_SESSION, sessionInformation);
+            }
             // Should we set properties from passed session?
             // imapStore.getServiceSession().getProperties().putAll(imapSession.getProperties());
             // imapStore.setPropagateClientIpAddress(imapSession.getProperty("mail.imap.propagate.clientipaddress"));
